/***************************************************************************//**
 *   @file   AD7730.h
 *   @brief  Header file of AD7730 Driver.
 *   @author DNechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 544
*******************************************************************************/
#ifndef _AD7730_H_
#define _AD7730_H_

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "Communication.h"

/******************************************************************************/
/* AD7730L                                                                    */
/******************************************************************************/

/* AD7730L GPIO */
#define AD7730L_RESET_OUT        GPIO1_PIN_OUT
#define AD7730L_RESET_LOW        GPIO1_LOW
#define AD7730L_RESET_HIGH       GPIO1_HIGH
#define AD7730L_SYNC_OUT         GPIO2_PIN_OUT
#define AD7730L_SYNC_LOW         GPIO2_LOW
#define AD7730L_SYNC_HIGH	     GPIO2_HIGH
#define AD7730L_READY_IN         GPIO3_PIN_IN
#define AD7730L_READY_STATUS     GPIO3_PIN_STATUS

/* AD7730L Register Map */
#define AD7730L_REG_COMM                  0 // Communications Register (WO, 8-bit)
#define AD7730L_REG_STAT                  0 // Status Register         (RO, 8-bit)
#define AD7730L_REG_DATA                  1 // Data Register           (RO, 16/24-bit)
#define AD7730L_REG_MODE                  2 // Mode Register           (RW, 16-bit)
#define AD7730L_REG_FILTER                3 // Filter Register         (RW, 24-bit)
#define AD7730L_REG_DAC                   4 // DAC Register            (RW, 8-bit)
#define AD7730L_REG_OFFSET                5 // Offset Register         (RW, 24-bit)
#define AD7730L_REG_GAIN                  6 // Gain Register           (RW, 24-bit)
#define AD7730L_REG_TEST                  7 // Test Register.          (RW, 24-bit)

/* Communications Register Bit Designations (AD7730L_REG_COMMUNICATION) */
#define AD7730L_COMM_WEN        (1 << 7)            // Write Enable.
#define AD7730L_COMM_RW(x)      (((x) & 0x03) << 4) // Read/Write Mode Bits.
#define AD7730L_COMM_ADDR(x)    ((x) & 0x07)        // Register Address.

/* Communication Register: AD7730L_COMM_RW(x) options */
#define AD7730L_COMM_SINGLE_WRITE      0 // Single Write to a Register.
#define AD7730L_COMM_SINGLE_READ       1 // Single Read of a Register.
#define AD7730L_COMM_START_CONT_RD     2 // Start Continuous Read of a Register. 
#define AD7730L_COMM_STOP_CONT_RD      3 // Stop Continuous Read Mode.

/* Status Register Bit Designations (AD7730L_REG_STAT) */
#define AD7730L_STAT_RDY              (1 << 7) // Ready Bit.
#define AD7730L_STAT_STDY             (1 << 6) // Steady Bit.
#define AD7730L_STAT_STBY             (1 << 5) // Standby Bit.
#define AD7730L_STAT_NOREF            (1 << 4) // No Reference Bit.
#define AD7730L_STAT_MS               0x0F     // These bits are for factory use.     

/* Mode Register Bit Designations (AD7730L_REG_MODE) */
#define AD7730L_MODE_SEL(x)         ((0x0007 & (unsigned short)(x)) << 13) // Mode Bits.
#define AD7730L_MODE_B_U            (1 << 12)                              // Bipolar/Unipolar Bit.
#define AD7730L_MODE_DEN            (1 << 11)                              // Digital Output Enable Bit.
#define AD7730L_MODE_D1             (1 << 10)                              // D1 - Digital Output Bit.
#define AD7730L_MODE_D0             (1 << 9)                               // D0 - Digital Output Bit.
#define AD7730L_MODE_WL             (1 << 8)                               // Data Word Length Bit.
#define AD7730L_MODE_HIREF          (1 << 7)                               // High Reference Bit.
#define AD7730L_MODE_RANGE_SEL(x)   ((0x0003 & (x)) << 4)                  // Input Range Bits.
#define AD7730L_MODE_CLKDIS         (1 << 3)                               // Master Clock Disable Bit.
#define AD7730L_MODE_BO             (1 << 2)                               // Burnout Current Bit.
#define AD7730L_MODE_CHANNEL_SEL(x) (0x0003 & (x))                         // Channel Selection Bits.

/* Mode Register: AD7730L_MODE_SEL options */
#define AD7730L_MODE_SYNC               0 // Sync (Idle) Mode.
#define AD7730L_MODE_CONT_CONV          1 // Continuous Conversion Mode.
#define AD7730L_MODE_SINGLE_CONV        2 // Single Conversion Mode.
#define AD7730L_MODE_POWER_DOWN         3 // Power-Down (Standby) Mode.
#define AD7730L_MODE_INT_ZERO_S_CLB     4 // Internal Zero-Scale Calibration.
#define AD7730L_MODE_INT_FULL_S_CLB     5 // Internal Full-Scale Calibration.
#define AD7730L_MODE_SYS_ZERO_S_CLB     6 // System Zero-Scale Calibration.
#define AD7730L_MODE_SYS_FULL_S_CLB     7 // System Full-Scale Calibration.

/* Mode Register: AD7730L_MODE_RANGE_SELECT options */
//                                           B/U Bit = 0        B/U Bit = 1
#define AD7730L_MODE_RANGE_10MV         0 // -10mV to +10mV    0mV to +10mV
#define AD7730L_MODE_RANGE_20MV         1 // -20mV to +20mV    0mV to +20mV
#define AD7730L_MODE_RANGE_40MV         2 // -40mV to +40mV    0mV to +40mV
#define AD7730L_MODE_RANGE_80MV         3 // -80mV to +80mV    0mV to +80mV

/* Mode Register: AD7730L_MODE_CHANNEL_SELECT options */
#define AD7730L_MODE_CH_AIN1P_AIN1M     0 // AIN1(+) AIN1(-)
#define AD7730L_MODE_CH_AIN2P_AIN2M     1 // AIN2(+) AIN2(-)
#define AD7730L_MODE_CH_AIN1M_AIN1M     2 // AIN1(-) AIN1(-)
#define AD7730L_MODE_CH_AIN1M_AIN2M     3 // AIN1(-) AIN2(-)


/* Filter Register Bit Designations (AD7730L_REG_FILTER) */
#define AD7730L_FILTER_SINC3_FILTER(x)  ((0x0FFF & (unsigned long)(x)) << 12) // Sinc3 Filter Selection Bits.
#define AD7730L_FILTER_SKIP             (1 << 9)                              // FIR Filter Skip Bit.
#define AD7730L_FILTER_FAST             (1 << 8)                              // FASTStep Mode Enable Bit.
#define AD7730L_FILTER_AC               (1 << 5)                              // AC Excitation Bit.
#define AD7730L_FILTER_CHP              (1 << 4)                              // Chop Enable Bit.
#define AD7730L_FILTER_DELAY_SELECT(x)  (0x000F & (x))                        // Delay Selection Bits.

/* DAC Register Bit Designations (AD7730L_REG_DAC) */
#define AD7730L_DAC_OFFSET_SEL(x)       ((x) * 0x3F) // Code for the offset-compensation DAC on the part.

/******************************************************************************/
/* Functions Prototypes                                                       */
/******************************************************************************/
/* Configures I/O pins from/towards the component and resets the device. */
unsigned char AD7730L_Init(void);

/* Resets the device. */
void AD7730L_SoftReset(void);

/* Writes data into a register. */
void AD7730L_SetRegisterValue(unsigned char registerAddress,
                              unsigned long registerValue,
                              unsigned char bytesNumber);
/* Reads the value of a register. */
unsigned long AD7730L_GetRegisterValue(unsigned char registerAddress,
                              unsigned char bytesNumber);
/* Waits for Ready pin to go low. */
void AD7730L_WaitForReady(void);

/* Writes the specified mode of operation in Mode Register. */
void AD7730L_SetMode(unsigned short modeOperation, 
                     unsigned char inputRange,
                     unsigned char channel);
/* Writes a value corresponding to an output rate in Filter Register. */
void AD7730L_SetFilterOutputRate(unsigned long outputRate,
                                 unsigned char chop,
                                 unsigned char skip);

/* Performs a calibration sequence for a channel. */
void AD7730L_Calibrate(unsigned char channel,
                       unsigned char inputRange,
                       unsigned long outputRate,
                       unsigned char dacValue
                       );

/* Reads the result of a single conversion. */
unsigned long AD7730L_SingleRead(unsigned char channel,
                                 unsigned char inputRange);

/* Returns the average of several conversion results. */
unsigned long AD7730L_ContinuousReadAvg(unsigned char channel,
                                        unsigned char inputRange,
                                        unsigned char sampleNumber);



#endif // _AD7730_H
